/* Children.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef CHILDREN_H_
#define CHILDREN_H_
#include <cstdarg>
#include <iostream>
#include <list>
#include <cstdlib>
#include "GL/glut.h"
#include "Node.h"
using std::cout;
using std::list;
#define children new Children

/**
 * Szenengraph-Node, der als T-Stück agiert und mehr als einen Kind-Knoten haben kann.
 */
class Children : public Node {
private:
	/**
	 * Die Liste der Kind-Knoten
	 */
	list<Node*> children_;

public:
	/**
	 * Liefert den n-ten Node
	 */
	Node* getNode(int index);

	/**
	 * Einen neuen Kind-Knoten zufügen
	 */
	void addNode(Node* nde);

	/**
	 * Löscht den letzten Node
	 */
	void removeNode();

	/**
	 * Den Baum traversieren: Hier werden work()-Methoden der Kind-Knoten aufgerufen
	 */
	void work();

	/**
	 * Liefert die Anzahl der Kind-Knoten
	 */
	int size();

	/**
	 * Default-Konstruktor: Wenn später Kinder zugefügt werden sollen
	 */
	Children();

	/**
	 * Konstruktor für bis zu 15 Kind-Knoten. Etwas unelegant, aber im Gegensatz zu einer
	 * Lösung per stdarg werden hier weder ein Anzahl-Parameter noch ein Sentinel gebraucht.
	 */
	Children(Node* n1, Node* n2 = 0, Node* n3 = 0, Node* n4 = 0, Node* n5 = 0,
			Node* n6 = 0, Node* n7 = 0, Node* n8 = 0, Node* n9 = 0, Node* n10 = 0,
			Node* n11 = 0, Node* n12 = 0, Node* n13 = 0, Node* n14 = 0, Node* n15 = 0);

	/**
	 * Default-Destruktor
	 */
	virtual ~Children();
};

#endif /*CHILDREN_H_*/
