/* Object.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef OBJECT_H_
#define OBJECT_H_
#include <iostream>
#include <cstdlib>
#include <GL/glut.h>
#include "Vector.h"
#include "Node.h"
#include "Material.h"
#include "Text.h"
using std::cout;

/**
 * Abstraktes Object-Node, das als Blatt im Baum des Szenengraphen auftritt.
 * Dieses sind die eigentlichen Objekte, die irgendetwas Zeichnen.
 */
class Object : public Node {
protected:
	/**
	 * Das Material ist aus Gründen der Objektorientierung eine Exemplarvariable und kein
	 * übergeordneter Knoten im Baum des Szenengraphen, auch wenn OpenGL Materialeigenschaften
	 * "einschaltet" vor Objekten.
	 */
	Material* mat;

	/**
	 * Optionales Text-Objekt (für Namen)
	 */
	Text* name;

	/**
	 * Variable um 3d-Koordinaten in 2d-Koordinaten zu verwandeln
	 */
	GLdouble x;

	/**
	 * Variable um 3d-Koordinaten in 2d-Koordinaten zu verwandeln
	 */
	GLdouble y;

	/**
	 * Variable um 3d-Koordinaten in 2d-Koordinaten zu verwandeln
	 */
	GLdouble z;

	/**
	 * Platz für die Modelview-Matrix
	 */
	GLdouble mm[16];

	/**
	 * Platz für die Projection-Matrix
	 */
	GLdouble pm[16];

	/**
	 * Platz für den Viewport
	 */
	GLint viewport[4];

public:
	/**
	 * Konstruktor mit optionalem Material
	 */
	Object(Material* mat = 0);

	/**
	 * Konstruktor mit Material und Namen
	 */
	Object(Material* mat, Text* name);

	/**
	 * Liefert das Material zurück
	 */
	Material* getMaterial();

	/**
	 * Methode, die die konkrete Zeichenroutine aufruft
	 */
	virtual void work();

	/**
	 * Abstrakte Methode zum Zeichnen des Objektes
	 */
	virtual void draw() = 0;

	/**
	 * Default-Destruktor
	 */
	virtual ~Object();
};

#endif /*OBJECT_H_*/
