/* Polygon.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FACESET_H_
#define FACESET_H_
#include <list>
#include <iostream>
#include <cstdarg>
#include "Object.h"
#include "Vector.h"
using std::list;
using std::cout;
#define polygon new Polygon
#define POLY_END 1000.0

/**
 * Konkretes Object-Node, das ein Polygon zeichnet.
 * Die Koordinaten des Polygons werden in einer Liste von double-Werten dem Konstruktor
 * übergeben, der eine variable Anzahl von Parametern liest. Die Anzahl muss x+1 betragen
 * mit (x mod 3)==0 und das (x+1).te Element muss POLY_END sein (Sentinel für variable
 * Parameteranzahl)
 */
class Polygon : public Object { 
private:
	/**
	 * Die Liste der Koordinaten
	 */
	list<double> coords;
public:
	/**
	 * Konstruktor für Polygon mit Material
	 */
	Polygon(Material* mat, ...);
	
	/**
	 * Konstruktor für Polygon ohne Material
	 */
	Polygon(double n1, ...);
	
	/** 
	 * Methode zum Zeichnen des Polygons
	 */
	void draw();
	
	/**
	 * Default-Destruktor
	 */
	virtual ~Polygon();
};

#endif /*FACESET_H_*/
