/* Rotation.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef ROTATION_H_
#define ROTATION_H_
#include "Transform.h"
#define rotation new Rotation

/**
 * Rotation ist ein Transform-Node und führt ein glRotate aus, bevor er den
 * Szenengraphen weiter traversiert und work() auf seinem Kind-Knoten ausführt.
 */
class Rotation : public Transform {
public:
	/**
	 * Konstruktor mit einem Koordinatenvektor und einem optionalen Kindknoten.
	 * Bei dem Vektor ist X der Winkel und Y, Z und W sind die Rotationsachsenwerte.
	 */
	Rotation(Vector* transformation, Node* child = 0);
	
	/**
	 * Konstruktor für vier Double-Werte und einem optionalem Kindknoten.
	 * a ist der Winkel und b, c und d sind die Rotationsachsenwerte.
	 */
	Rotation(double a, double b, double c, double d, Node* child = 0);
	
	/**
	 * Baum weiter traversieren, d.h. work() auf Kindknoten ausführen
	 */
	void work();
	
	/**
	 * Default-Destruktor
	 */
	virtual ~Rotation();
};

#endif /*ROTATION_H_*/
