/* Scale.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef SCALE_H_
#define SCALE_H_
#include "Transform.h"
#define scale new Scale

/**
 * Scale ist ein Transform-Node und führt ein glScale aus, bevor er den
 * Szenengraphen weiter traversiert und work() auf seinem Kind-Knoten ausführt.
 */
class Scale : public Transform  {
public:
	/**
	 * Konstruktor für einen Vektor, von dem die ersten drei Werte als Skalierungswerte
	 * verwendet werden und einem optionale Kindknoten
	 */
	Scale(Vector* transformation, Node* child = 0);
	
	/**
	 * Konstruktor für drei Double-Werte als Skalierungswerte und einem optionalen
	 * Kindknoten
	 */
	Scale(double a, double b, double c, Node* child = 0);
	
	/**
	 * Baum weiter traversieren, d.h. work() auf Kindknoten ausführen
	 */
	void work();
	
	/**
	 * Default-Destruktor
	 */
	virtual ~Scale();
};

#endif /*SCALE_H_*/
