/* Shark.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef SHARK_H_
#define SHARK_H_

#include "Fish.h"

/**
 * Hai: Erbt von Fisch, nutzt seine Eigenschaften und Vertex- und Normalendefintionen,
 * bewegt sich auf den Fischschwarm zu
 */
class Shark : public Fish {
private:
	/**
	 * Welcher Fisch wird momentan verfolgt?
	 */
	Fish* target;
	
public:
	/**
	 * Konstruktor mit Material, Name, Position und Richtung
	 */
	Shark(Material* mat, Text* name, Vector* position, Vector* direction);
	
	/**
	 * Zeichnet den Hai
	 */
	virtual void draw();
	
	/**
	 * Bewegt den Hai. Liefert true, wenn noch kein Fisch gefressen wurde, und false,
	 * wenn der letzte Fisch der Fischliste gefressen wurde
	 */
	virtual bool move();
	
	/**
	 * Sucht ein neues Ziel: Den letzten Fisch der Fischliste, nicht jeden den Leuchtfisch
	 * (der hat diplomatische Immunität)
	 */
	void nextTarget();
	
	/**
	 * Default-Destruktor
	 */
	virtual ~Shark();
};

#endif /*SHARK_H_*/
