/* Text.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef TEXT_H_
#define TEXT_H_
#include <list>
#include <string>
#include <cstdlib>
#include <GL/glut.h>
#include "Textures.h"
using std::list;
using std::string;

/**
 * Text-Objekt: Hat einen Text, eine Skalierung und eine Position (2d)
 */
class Text {
private:
	/**
	 * X-Koordinate
	 */
	int x;

	/**
	 * Y-Koordinate
	 */
	int y;

	/**
	 * Anzuzeigender Text
	 */
	string text;

	/**
	 * Skalierung (Schriftgröße)
	 */
	float scale;

	/**
	 * Zeit an, ob dieses Textobjekt gezeichnet werden soll oder nicht
	 */
	bool enabled;

	/**
	 * Statische Liste aller Textobjekte
	 */
	static list<Text*> texts;

public:
	/**
	 * Konstruktor mit Koordinaten, Text und optionaler Skalierung
	 */
	Text(int x, int y, string text, float scale = 1.0);

	/**
	 * Konstruktor mit Text und optionaler Skalierung
	 */
	Text(string text, float scale = 1.0);

	/**
	 * Koordinaten ändern
	 */
	void setCoords(int x, int y);

	/**
	 * Darstellung an- oder abstellen
	 */
	void setEnabled(bool enabled);

	/**
	 * Text ändern
	 */
	void setText(string text);

	/**
	 * Alle Textobjekte Zeichnen
	 */
	static void drawAll();

	/**
	 * Nur ein Objekt zeichnen
	 */
	void draw();

	/**
	 * Default-Destruktor
	 */
	virtual ~Text();
};

#endif /*TEXT_H_*/
