/* Textures.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef TEXTURES_H_
#define TEXTURES_H_
#include <string>
#include <iostream>
#include <cstdio>
#include <GL/glut.h>
#include "config.h"
using std::string;
using std::cout;
using std::cerr;

#define NUMTEX 6

/**
 * Klasse zum Laden und Verwalten von Texturen.
 * Stellt ausserdem eine Methode zum Zeichnen von Text zur Verfügung, da diese technisch
 * sehr nah mit der Texturenverwaltung verbandelt ist
 */
class Textures {
private:
	/**
	 * Speichert den Anfang der Displaylist für die Schriftausgabe
	 */
	static GLuint fontbase;

	/**
	 * Textur-Array
	 */
	static GLuint tex[NUMTEX];

	/**
	 * Lädt eine Textur - eine .bmp-Datei - in eine Nummer des Textur-Arrays
	 */
	static void loadTexture(string &filename, int number);

	/**
	 * Konstruiert die Displaylist für die Schriftausgabe aus der Textur
	 */
	static void buildFont();

	/**
	 * Methode zur Ausgabe von Text. Koordinaten und Skalierung sind relativ zur in der
	 * Datei config.h angegebenen Auflösung. Die untere linke Ecke hat dabei die Koordinaten
	 * (0, 0), und die obere rechte Ecke (800, 600).
	 */
	static void glPrint(GLint x, GLint y, const char* string, float scale);

	/**
	 * Privater Default-Konstruktor: Alle eigentlichen Funktionen sind Klassenmethoden.
	 */
	Textures();

public:
	/**
	 * Texturkonstante
	 */
	static int FONT;

	/**
	 * Texturkonstante
	 */
	static int WOOD1;

	/**
	 * Texturkonstante
	 */
	static int WOOD2;

	/**
	 * Texturkonstante
	 */
	static int SAND;

	/**
	 * Texturkonstante
	 */
	static int PLANT;

	/**
	 * Texturkonstante
	 */
	static int ENVMAP;

	/**
	 * Texturen laden, Schrift bauen
	 */
	static void init();

	/**
	 * Eine Textur aus dem Array aktivieren
	 */
	static void select(int t);

	/**
	 * Wrapper-Methode zur Ausgabe von Text: Zeichnet automatisch Schatten usw.
	 */
	static void print(GLint x, GLint y, const char* string, float scale);

	/**
	 * Default-Destruktor
	 */
	virtual ~Textures();
};

#endif /*TEXTURES_H_*/
