/* Transform.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef TRANSFORM_H_
#define TRANSFORM_H_
#include <cstdlib>
#include <GL/glut.h>
#include "Node.h"
#include "Vector.h"

/**
 * Abstrakter Knoten im Szenengraph, der eine Transformation durchführt
 */
class Transform : public Node {
protected:
	/**
	 * Vektor, der die Transformationsdaten Speichert
	 */
	Vector* transformation;

public:
	/**
	 * Default-Konstruktor
	 */
	Transform();

	/**
	 * Konstruktor, der einen Vektor mit Transformationsdaten liest
	 */
	Transform(Vector* transformation);

	/**
	 * Konstruktor, der vier Double-Werte als Transformationsdaten liest und daraus
	 * den zur internen Speicherung verwendeten Vektor generiert
	 */
	Transform(double a, double b = 0.0, double c = 0.0, double d = 0.0);

	/**
	 * Konstruktor, der einen Vektor mit Transformationsdaten und einen optionalen
	 * Kindknoten liest
	 */
	Transform(Vector* transformation, Node* child);

	/**
	 * Konstruktor, der vier Double-Werte als Transformationsdaten liest und daraus
	 * den zur internen Speicherung verwendeten Vektor generiert, sowie einen optionalen
	 * Kindknoten
	 */
	Transform(double a, double b, double c, double d, Node* child);

	/**
	 * Liefert den aktuellen Transformationsdatenvektor
	 */
	Vector* getTransformation();

	/**
	 * Muss von tatsächlicher Transformation implementiert werden, führt diese durch
	 */
	void work() = 0;

	/**
	 * Default-Destruktor
	 */
	virtual ~Transform();
};

#endif /*TRANSFORM_H_*/
