/* Vector.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef COORD3D_H_
#define COORD3D_H_
#include <cstdlib>
#include <GL/glut.h>
#include <iostream>
#include <cmath>
using std::cout;
using std::ostream;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define vector new Vector
#define color new Vector

/**
 * Vektorklasse zur Speicherung von eines Viertupels. Wird als solches auch missbraucht,
 * um z.B. Zahlenwerte abzulegen. Die Komponenten sind als public deklariert, aus Gründen
 * der einfacheren Verwendung.
 */
class Vector {
public:
	/**
	 * X-Komponente
	 */
	double x;

	/**
	 * X-Komponente
	 */
	double y;

	/**
	 * Y-Komponente
	 */
	double z;

	/**
	 * W-Komponente
	 */
	double w;

	/**
	 * Vektor der die X-Achse beschreibt
	 */
	static Vector e1;

	/**
	 * Vektor der die Y-Achse beschreibt
	 */
	static Vector e2;

	/**
	 * Vektor der die X-Achse beschreibt
	 */
	static Vector e3;

	/**
	 * Konstruktor für vier Int-Werte, die automatisch zu Double konvertiert werden
	 */
	Vector(int a, int b = 0, int c = 0, int d = 1);

	/**
	 * Konstruktor für vier Double-Werte
	 */
	Vector(double a = 0.0, double b = 0.0, double c = 0.0, double d = 1.0);

	/**
	 * Ausgabe als Outputstream
	 */
	friend ostream& operator<<(ostream& o, Vector& v);

	/**
	 * Berechnet den Winkel in Draufsicht der beiden Vektoren
	 */
	const double angle2dxz(const Vector& v);

	/**
	 * Kreuzprodukt
	 */
	const Vector operator%(const Vector& v);

	/**
	 * Vektormultiplikation
	 */
	const void operator*=(const GLfloat& v);

	/**
	 * Vektormultiplikation
	 */
	const Vector operator*(const GLfloat& v);

	/**
	 * Vektoraddition
	 */
	const Vector operator+(const Vector& v);

	/**
	 * Vektoraddition
	 */
	const void operator+=(const Vector& v);

	/**
	 * Vektorsubtraktion
	 */
	const Vector operator-(const Vector& v);

	/**
	 * Vektorsubtraktion
	 */
	const void operator-=(const Vector& v);

	/**
	 * Länge des Vektors
	 */
	const double length() const;

	/**
	 * Skalarprodukt
	 */
	const double dot(const Vector& v);

	/**
	 * Vektor normieren
	 */
	void norm();

	/**
	 * Setter-Methode, zum einfachen Setzen aller vier Komponenten auf einmal, mit
	 * automatischer Int->Double-Konvertierung
	 */
	void set(int a, int b, int c, int d = 1);

	/**
	 * Setter-Methode, zum einfachen Setzen aller vier Komponenten auf einmal
	 */
	void set(double a, double b, double c, double d = 1.0);

	/**
	 * Default-Destruktor
	 */
	virtual ~Vector();
};

#define Color Vector

#endif /*COORD3D_H_*/
